@@ -406,7 +406,7 @@ class JsonResource(Resource):
                 raise AmbiguousAssetError('Data get must end with *', json_path)
             json_path = json_path[:-2]
 
-            result = self.get_jsonpath(json_path)
+            result = self.get_jsonpath(json_path, default=[])
 
             if isinstance(result, dict):
                 for key in result.keys():
@@ -765,6 +765,7 @@ class ResourcePack(Pack): pass
 class ResourcePack(Pack):
     def __init__(self, input_path: str, project: Project = None):
         super().__init__(input_path, project=project)
+        self.__particles = []
         self.__attachables = []
         self.__animation_controller_files = []
         self.__animation_files = []
@@ -774,6 +775,15 @@ class ResourcePack(Pack):
         self.__items = []
         
     
+    @cached_property
+    def particles(self) -> list[ParticleFileRP]:
+        base_directory = os.path.join(self.input_path, "particles")
+        for local_path in glob.glob(base_directory + "/**/*.json", recursive=True):
+            local_path = os.path.relpath(local_path, self.input_path)
+            self.__particles.append(ParticleFileRP(file_path = local_path, pack = self))
+            
+        return self.__particles
+
     @cached_property
     def attachables(self) -> list[AttachableFileRP]:
         base_directory = os.path.join(self.input_path, "attachables")
@@ -863,6 +873,12 @@ class ResourcePack(Pack):
         return children
 
     
+    def get_particle(self, identifier:str) -> ParticleFileRP:
+        for child in self.particles:
+            if child.identifier == identifier:
+                return child
+        raise AssetNotFoundError(identifier)
+
     def get_attachable(self, identifier:str) -> AttachableFileRP:
         for child in self.attachables:
             if child.identifier == identifier:
@@ -893,6 +909,12 @@ class ResourcePack(Pack):
                 return child
         raise AssetNotFoundError(file_name)
 
+    def get_render_controller_file(self, file_name:str) -> RenderControllerFileRP:
+        for child in self.render_controller_files:
+            if child.file_name == file_name:
+                return child
+        raise AssetNotFoundError(file_name)
+
     
     def get_render_controller(self, id:str) -> RenderControllerRP:
         for file_child in self.render_controller_files:
@@ -1051,6 +1073,74 @@ class BehaviorPack(Pack):
 
     
     
+class RecipeFile(JsonFileResource):
+    def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
+        super().__init__(data = data, file_path = file_path, pack = pack)
+        
+    
+    @property
+    def identifier(self):
+        return self.get_jsonpath("**/identifier")
+    
+    @identifier.setter
+    def identifier(self, identifier):
+        return self.set_jsonpath("**/identifier", identifier)
+
+    
+    
+    
+    
+class ParticleFileRP(JsonFileResource):
+    def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
+        super().__init__(data = data, file_path = file_path, pack = pack)
+        self.__components = []
+        self.__events = []
+        
+    
+    @property
+    def format_version(self):
+        return self.get_jsonpath("format_version")
+    
+    @format_version.setter
+    def format_version(self, format_version):
+        return self.set_jsonpath("format_version", format_version)
+
+    @property
+    def identifier(self):
+        return self.get_jsonpath("particle_effect/description/identifier")
+    
+    @identifier.setter
+    def identifier(self, identifier):
+        return self.set_jsonpath("particle_effect/description/identifier", identifier)
+
+    
+    @cached_property
+    def components(self) -> list[JsonSubResource]:
+        for path, data in self.get_data_at("particle_effect/components/*"):
+            self.__components.append(JsonSubResource(parent = self, json_path = path, data = data))
+        return self.__components
+    
+    @cached_property
+    def events(self) -> list[JsonSubResource]:
+        for path, data in self.get_data_at("particle_effect/events/*"):
+            self.__events.append(JsonSubResource(parent = self, json_path = path, data = data))
+        return self.__events
+    
+    
+    def get_component(self, id:str) -> JsonSubResource:
+        for child in self.components:
+            if child.id == id:
+                return child
+        raise AssetNotFoundError(id)
+
+    def get_event(self, id:str) -> JsonSubResource:
+        for child in self.events:
+            if child.id == id:
+                return child
+        raise AssetNotFoundError(id)
+
+    
+    
 class AttachableFileRP(JsonFileResource):
     def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
         super().__init__(data = data, file_path = file_path, pack = pack)
@@ -1118,7 +1208,7 @@ class RenderControllerFileRP(JsonFileResource):
     
     @cached_property
     def render_controllers(self) -> list[RenderControllerRP]:
-        for path, data in self.get_jsonpath("render_controllers/*"):
+        for path, data in self.get_data_at("render_controllers/*"):
             self.__render_controllers.append(RenderControllerRP(parent = self, json_path = path, data = data))
         return self.__render_controllers
     
@@ -1137,10 +1227,18 @@ class AnimationControllerFileRP(JsonFileResource):
         self.__animation_controllers = []
         
     
+    @property
+    def format_version(self):
+        return self.get_jsonpath("format_version")
+    
+    @format_version.setter
+    def format_version(self, format_version):
+        return self.set_jsonpath("format_version", format_version)
+
     
     @cached_property
     def animation_controllers(self) -> list[AnimationControllerRP]:
-        for path, data in self.get_jsonpath("animation_controllers/*"):
+        for path, data in self.get_data_at("animation_controllers/*"):
             self.__animation_controllers.append(AnimationControllerRP(parent = self, json_path = path, data = data))
         return self.__animation_controllers
     
@@ -1187,7 +1285,7 @@ class LootTableFile(JsonFileResource):
     
     @cached_property
     def pools(self) -> list[LootTablePool]:
-        for path, data in self.get_jsonpath("pools/*"):
+        for path, data in self.get_data_at("pools/*"):
             self.__pools.append(LootTablePool(parent = self, json_path = path, data = data))
         return self.__pools
     
@@ -1219,7 +1317,7 @@ class ItemFileRP(JsonFileResource):
     
     @cached_property
     def components(self) -> list[Component]:
-        for path, data in self.get_jsonpath("minecraft:item/components/*"):
+        for path, data in self.get_data_at("minecraft:item/components/*"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
     
@@ -1243,7 +1341,7 @@ class ItemFileBP(JsonFileResource):
     
     @cached_property
     def components(self) -> list[Component]:
-        for path, data in self.get_jsonpath("minecraft:item/components"):
+        for path, data in self.get_data_at("minecraft:item/components"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
     
@@ -1267,7 +1365,7 @@ class BlockFileBP(JsonFileResource):
     
     @cached_property
     def components(self) -> list[Component]:
-        for path, data in self.get_jsonpath("minecraft:block/components"):
+        for path, data in self.get_data_at("minecraft:block/components"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
     
@@ -1291,7 +1389,7 @@ class EntityFileRP(JsonFileResource):
     
     @cached_property
     def animations(self) -> list[AnimationRP]:
-        for path, data in self.get_jsonpath("minecraft:client_entity/description/animations/*"):
+        for path, data in self.get_data_at("minecraft:client_entity/description/animations/*"):
             self.__animations.append(AnimationRP(parent = self, json_path = path, data = data))
         return self.__animations
     
@@ -1315,7 +1413,7 @@ class AnimationFileRP(JsonFileResource):
     
     @cached_property
     def animations(self) -> list[AnimationRP]:
-        for path, data in self.get_jsonpath("animations/*"):
+        for path, data in self.get_data_at("animations/*"):
             self.__animations.append(AnimationRP(parent = self, json_path = path, data = data))
         return self.__animations
     
@@ -1349,19 +1447,19 @@ class EntityFileBP(JsonFileResource):
     
     @cached_property
     def component_groups(self) -> list[ComponentGroup]:
-        for path, data in self.get_jsonpath("minecraft:entity/component_groups/*"):
+        for path, data in self.get_data_at("minecraft:entity/component_groups/*"):
             self.__component_groups.append(ComponentGroup(parent = self, json_path = path, data = data))
         return self.__component_groups
     
     @cached_property
     def components(self) -> list[Component]:
-        for path, data in self.get_jsonpath("minecraft:entity/components/*"):
+        for path, data in self.get_data_at("minecraft:entity/components/*"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
     
     @cached_property
     def events(self) -> list[Event]:
-        for path, data in self.get_jsonpath("minecraft:entity/events/*"):
+        for path, data in self.get_data_at("minecraft:entity/events/*"):
             self.__events.append(Event(parent = self, json_path = path, data = data))
         return self.__events
     
@@ -1380,13 +1478,13 @@ class EntityFileBP(JsonFileResource):
 
     
     def create_component_group(self, name: str, data: dict) -> ComponentGroup:
-        self.set_jsonpath("minecraft:entity/component_groups/." + name, data)
+        self.set_jsonpath("minecraft:entity/component_groups/" + name, data)
         new_object = ComponentGroup(self, "minecraft:entity/component_groups/." + name, data)
         self.__component_groups.append(new_object)
         return new_object
 
     def create_component(self, name: str, data: dict) -> Component:
-        self.set_jsonpath("minecraft:entity/components/." + name, data)
+        self.set_jsonpath("minecraft:entity/components/" + name, data)
         new_object = Component(self, "minecraft:entity/components/." + name, data)
         self.__components.append(new_object)
         return new_object
@@ -1401,29 +1499,13 @@ class ModelFileRP(JsonFileResource):
     
     @cached_property
     def models(self) -> list[Model]:
-        for path, data in self.get_jsonpath("minecraft:geometry/*"):
+        for path, data in self.get_data_at("minecraft:geometry/*"):
             self.__models.append(Model(parent = self, json_path = path, data = data))
         return self.__models
     
     
     
     
-class AnimationControllerFile(JsonFileResource):
-    def __init__(self, data: dict = None, file_path: str = None, pack: Pack = None) -> None:
-        super().__init__(data = data, file_path = file_path, pack = pack)
-        self.__animation_controllers = []
-        
-    
-    
-    @cached_property
-    def animation_controllers(self) -> list[AnimationController]:
-        for path, data in self.get_jsonpath("animation_controllers/*"):
-            self.__animation_controllers.append(AnimationController(parent = self, json_path = path, data = data))
-        return self.__animation_controllers
-    
-    
-    
-    
 class RenderControllerRP(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
@@ -1434,7 +1516,7 @@ class RenderControllerRP(JsonSubResource):
     
     
     
-class AnimationControllerRP(JsonSubResource):
+class Cube(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
         
@@ -1454,7 +1536,7 @@ class LootTablePool(JsonSubResource):
     
     
     
-class AnimationControllerState(JsonSubResource):
+class AnimationControllerStateRP(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
         
@@ -1464,6 +1546,37 @@ class AnimationControllerState(JsonSubResource):
     
     
     
+class AnimationControllerRP(JsonSubResource):
+    def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
+        super().__init__(data=data, parent=parent, json_path=json_path)
+        
+        self.__states = []
+        
+    
+    @cached_property
+    def states(self) -> list[AnimationControllerStateRP]:
+        for path, data in self.get_data_at("states/*"):
+            self.__states.append(AnimationControllerStateRP(parent = self, json_path = path, data = data))
+        return self.__states
+    
+    
+    @property
+    def initial_state(self):
+        return self.get_jsonpath("initial_state")
+    
+    @initial_state.setter
+    def initial_state(self, initial_state):
+        return self.set_jsonpath("initial_state", initial_state)
+
+    
+    def get_state(self, id:str) -> AnimationControllerStateRP:
+        for child in self.states:
+            if child.id == id:
+                return child
+        raise AssetNotFoundError(id)
+
+    
+    
 class Model(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
@@ -1473,7 +1586,7 @@ class Model(JsonSubResource):
     
     @cached_property
     def bones(self) -> list[Bone]:
-        for path, data in self.get_jsonpath("bones/*"):
+        for path, data in self.get_data_at("bones/*"):
             self.__bones.append(Bone(parent = self, json_path = path, data = data))
         return self.__bones
     
@@ -1493,26 +1606,24 @@ class AnimationRP(JsonSubResource):
     def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
         super().__init__(data=data, parent=parent, json_path=json_path)
         
-        
-    
-    
-    
-    
-    
-class AnimationController(JsonSubResource):
-    def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
-        super().__init__(data=data, parent=parent, json_path=json_path)
-        
-        self.__states = []
+        self.__bones = []
         
     
     @cached_property
-    def states(self) -> list[AnimationControllerState]:
-        for path, data in self.get_jsonpath("states/*"):
-            self.__states.append(AnimationControllerState(parent = self, json_path = path, data = data))
-        return self.__states
+    def bones(self) -> list[JsonSubResource]:
+        for path, data in self.get_data_at("bones"):
+            self.__bones.append(JsonSubResource(parent = self, json_path = path, data = data))
+        return self.__bones
     
     
+    @property
+    def loop(self):
+        return self.get_jsonpath("loop")
+    
+    @loop.setter
+    def loop(self, loop):
+        return self.set_jsonpath("loop", loop)
+
     
     
     
@@ -1525,7 +1636,7 @@ class ComponentGroup(JsonSubResource):
     
     @cached_property
     def components(self) -> list[Component]:
-        for path, data in self.get_jsonpath("*"):
+        for path, data in self.get_data_at("*"):
             self.__components.append(Component(parent = self, json_path = path, data = data))
         return self.__components
     
@@ -1533,7 +1644,7 @@ class ComponentGroup(JsonSubResource):
     
     
     def create_component(self, name: str, data: dict) -> Component:
-        self.set_jsonpath("." + name, data)
+        self.set_jsonpath("" + name, data)
         new_object = Component(self, "." + name, data)
         self.__components.append(new_object)
         return new_object
@@ -1559,13 +1670,13 @@ class Event(JsonSubResource):
     
     @cached_property
     def groups_to_add(self) -> list[ComponentGroup]:
-        for path, data in self.get_jsonpath("add/component_groups/*"):
+        for path, data in self.get_data_at("add/component_groups/*"):
             self.__groups_to_add.append(ComponentGroup(parent = self, json_path = path, data = data))
         return self.__groups_to_add
     
     @cached_property
     def groups_to_remove(self) -> list[ComponentGroup]:
-        for path, data in self.get_jsonpath("remove/component_groups/*"):
+        for path, data in self.get_data_at("remove/component_groups/*"):
             self.__groups_to_remove.append(ComponentGroup(parent = self, json_path = path, data = data))
         return self.__groups_to_remove
     
@@ -1582,21 +1693,11 @@ class Bone(JsonSubResource):
     
     @cached_property
     def cubes(self) -> list[Cube]:
-        for path, data in self.get_jsonpath("cubes/*"):
+        for path, data in self.get_data_at("cubes/*"):
             self.__cubes.append(Cube(parent = self, json_path = path, data = data))
         return self.__cubes
     
     
     
     
-    
-class Cube(JsonSubResource):
-    def __init__(self, data: dict = None, parent: Resource = None, json_path: str = None ) -> None:
-        super().__init__(data=data, parent=parent, json_path=json_path)
-        
-        
-    
-    
-    
-    
     
\ No newline at end of file
